use crate::geometry::{Point, Point2, Ray, Transform, Vector, ray, vec3};

#[derive(Debug, Clone)]
pub struct Camera {
    /// camera to world transform
    transform: Transform,
}

impl Camera {
    pub fn new(eye: Point, center: Point, up: Vector, fov: f32) -> Self {
        let f = (center - eye).length();
        let tan = (fov / 2.0).to_radians().tan();
        let transform =
            Transform::look_at(eye, center, up).inverse() * Transform::scale(tan * f, tan * f, f);

        Self { transform }
    }

    pub fn ray(&self, p_film: Point2) -> Ray {
        ray(
            self.transform * Point::ORIGIN,
            self.transform * vec3(p_film.x, p_film.y, 1.0),
        )
    }
}
