use std::fmt;
use std::ops::{Add, AddAssign, Div, DivAssign, Index, Mul, MulAssign};

use glam::Vec3A;

use crate::Vector;

#[derive(Clone, Copy)]
pub struct Rgb(Vec3A);

impl Rgb {
    pub const BLACK: Self = rgb(0.0, 0.0, 0.0);
}

impl num_traits::Zero for Rgb {
    fn zero() -> Self {
        Self(Vec3A::ZERO)
    }

    fn is_zero(&self) -> bool {
        self.0 == Vec3A::ZERO
    }
}

pub const fn rgb(r: f32, g: f32, b: f32) -> Rgb {
    Rgb(glam::vec3a(r, g, b))
}

impl From<Vector> for Rgb {
    fn from(v: Vector) -> Self {
        Self(v.into())
    }
}

impl fmt::Debug for Rgb {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple(stringify!(Rgb))
            .field(&self.0.x)
            .field(&self.0.y)
            .field(&self.0.z)
            .finish()
    }
}

impl Index<usize> for Rgb {
    type Output = f32;

    #[inline]
    fn index(&self, index: usize) -> &Self::Output {
        match index {
            0 => &self.0.x,
            1 => &self.0.y,
            2 => &self.0.z,
            _ => panic!("index out of bounds"),
        }
    }
}

impl Add for Rgb {
    type Output = Self;

    fn add(self, rhs: Self) -> Self::Output {
        Self(self.0 + rhs.0)
    }
}

impl AddAssign for Rgb {
    fn add_assign(&mut self, rhs: Self) {
        self.0 += rhs.0
    }
}

impl Mul<f32> for Rgb {
    type Output = Self;

    fn mul(self, rhs: f32) -> Self::Output {
        Self(self.0 * rhs)
    }
}

impl MulAssign<f32> for Rgb {
    fn mul_assign(&mut self, rhs: f32) {
        self.0 *= rhs
    }
}

impl Div<f32> for Rgb {
    type Output = Self;

    fn div(self, rhs: f32) -> Self::Output {
        Self(self.0 / rhs)
    }
}

impl DivAssign<f32> for Rgb {
    fn div_assign(&mut self, rhs: f32) {
        self.0 /= rhs
    }
}
