use crate::{
    Shape,
    geometry::{Hit, Hittable, Ray},
};

#[derive(Debug, Clone)]
pub struct Scene(Vec<Shape>);

impl Scene {
    pub fn new(objects: Vec<Shape>) -> Self {
        Self(objects)
    }

    pub fn intersect(&self, r: Ray) -> Option<Hit> {
        let mut hit: Option<Hit> = None;

        for obj in &self.0 {
            let t_max = hit.as_ref().map(|h| h.t).unwrap_or(f32::INFINITY);
            if let Some(h) = obj.intersect(&r, t_max) {
                hit = Some(h)
            }
        }

        hit
    }
}
